package bitsplease.mummymayhem;

import bitsplease.mummymayhem.model.Player;
import bitsplease.mummymayhem.model.buildings.Barrack;
import bitsplease.mummymayhem.model.buildings.Tower;
import bitsplease.mummymayhem.model.buildings.TowerType;
import bitsplease.mummymayhem.model.utils.ResourceLoader;
import javafx.fxml.FXML;
import javafx.geometry.Insets;
import javafx.geometry.Point2D;
import javafx.scene.Node;
import javafx.scene.control.*;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;

public class BuildDialogController {
    @FXML
    private VBox buildingList;

    private Player currentPlayer;

    /**
     * Create and load the Building Dialog window.
     */
    @FXML
    public void initialize() {
        currentPlayer = GameViewController.getModel().getActualPlayer();
        buildingList.getStyleClass().add("building-dialog");
        createTowerInfoSections();
        createBarrackInfoSection();
    }

    private void createBarrackInfoSection() {
        Barrack barrack = new Barrack(new Point2D(0,0));
        HBox buildingItem = new HBox();
        VBox buildingInfo = new VBox();
        Label nameLabel = new Label("Barrack");
        Label priceLabel = new Label("Price: " + Barrack.getPrice() + " g");
        Button buildButton = new Button("Build");
        ImageView buildingImage = createImgView(barrack.getImagePath());

        buildingItem.getStyleClass().add("building-item");
        buildingInfo.getStyleClass().add("building-info");
        buildButton.getStyleClass().add("action-button");
        buildButton.addEventHandler(MouseEvent.MOUSE_CLICKED, mouseEvent -> {
            GameViewController.getModel().selectBuilding(barrack);
            Node source = (Node) mouseEvent.getSource();
            Stage stage = (Stage) source.getScene().getWindow();
            stage.close();
        });
        buildButton.setDisable(Barrack.getPrice() > currentPlayer.getGold());
        buildingInfo.getChildren().addAll(nameLabel, priceLabel, buildButton);
        buildingItem.getChildren().addAll(buildingImage, buildingInfo);

        Separator hs = new Separator();
        buildingList.getChildren().addAll(buildingItem, hs);
    }

    private void createTowerInfoSections() {
        for(TowerType type : TowerType.values()) {
            HBox buildingItem = new HBox();
            VBox buildingInfo = new VBox();
            ImageView buildingImage = createImgView(type.getImagePath());
            Label rangeLabel = new Label("Range: " + type.getRange());
            Label damageLabel = new Label("Damage: " + type.getAttack());
            Label hpLabel = new Label("HP: " + type.getHp());
            Label priceLabel = new Label("Price: " + type.getPrice() + " g");
            Button buildButton = new Button("Build");
            buildButton.setDisable(type.getPrice() > currentPlayer.getGold());

            buildingItem.getStyleClass().add("building-item");
            buildingInfo.getStyleClass().add("building-info");
            buildButton.getStyleClass().add("action-button");
            buildButton.addEventHandler(MouseEvent.MOUSE_CLICKED, mouseEvent -> {
                GameViewController.getModel().selectBuilding(new Tower(type));
                Node source = (Node) mouseEvent.getSource();
                Stage stage = (Stage) source.getScene().getWindow();
                stage.close();
            });
            buildingInfo.getChildren().addAll(rangeLabel, damageLabel, hpLabel, priceLabel, buildButton);
            buildingItem.getChildren().addAll(buildingImage, buildingInfo);

            Separator hs = new Separator();
            hs.setPadding(new Insets(10, 20, 10, 20));
            hs.getStyleClass().add("separator-item");
            buildingList.getChildren().addAll(buildingItem, hs);
        }
    }

    /**
     * Create an ImageView according to the image path.
     * @param imagePath path of the image
     * @return an ImageView
     */
    public ImageView createImgView(String imagePath) {
        ImageView buildingImage = new ImageView();
        Image imgHolder = ResourceLoader.loadImage(imagePath);
        buildingImage.setImage(imgHolder);
        buildingImage.setFitWidth(100);
        buildingImage.setPreserveRatio(true);
        return buildingImage;
    }
}

