package bitsplease.mummymayhem;

import bitsplease.mummymayhem.model.utils.DialogDisplayer;
import bitsplease.mummymayhem.model.utils.SFXStorage;
import bitsplease.mummymayhem.model.utils.SFXType;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.input.MouseEvent;
import javafx.stage.Stage;

import java.io.IOException;
import java.util.Objects;
import java.util.Optional;


public class MainWindowController {

    /**
     * Load the Game View and set the scene.
     * @param event a MouseEvent object (containing the source)
     * @throws IOException if I/O error has occurred
     */
    @FXML
    protected void onNewGameButtonClicked(MouseEvent event) throws IOException {
        SFXStorage.getSFX(SFXType.BUTTON_CLICK).play();

        Node node = (Node) event.getSource();
        Stage thisStage = (Stage) node.getScene().getWindow();

        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("game-view.fxml"));
        Scene gameScene = new Scene(fxmlLoader.load());
        gameScene.getStylesheets().add(Objects.requireNonNull(getClass().getResource("mmstyle.css")).toExternalForm());

        GameViewController gameViewController = fxmlLoader.getController();
        thisStage.xProperty().addListener((obs, oldVal, newVal) -> gameViewController.movePopupsWithWindow(thisStage));
        thisStage.yProperty().addListener((obs, oldVal, newVal) -> gameViewController.movePopupsWithWindow(thisStage));

        thisStage.setScene(gameScene);
    }

    /**
     * Load the Map Editor View and set the scene.
     * @param event a MouseEvent object (containing the source)
     * @throws IOException if I/O error has occurred
     */
    @FXML
    protected void onMapEditorButtonClicked(MouseEvent event) throws IOException {
        Node node = (Node) event.getSource();
        Stage thisStage = (Stage) node.getScene().getWindow();

        FXMLLoader fxmlLoader = new FXMLLoader(getClass().getResource("mapeditor-view.fxml"));
        Scene mapEditorScene = new Scene(fxmlLoader.load());
        mapEditorScene.getStylesheets().add(Objects.requireNonNull(getClass().getResource("mmstyle.css")).toExternalForm());

        MapEditorController mapEditorController = fxmlLoader.getController();
        thisStage.xProperty().addListener((obs, oldVal, newVal) -> mapEditorController.movePopupWithWindow(thisStage));
        thisStage.yProperty().addListener((obs, oldVal, newVal) -> mapEditorController.movePopupWithWindow(thisStage));

        thisStage.setScene(mapEditorScene);
    }

    /**
     * Display a confirmation dialog, then exit the application or stay on the Main Window.
     */
    @FXML
    protected void onExitButtonClicked() {
        SFXStorage.getSFX(SFXType.BUTTON_CLICK).play();

        Optional<ButtonType> result = DialogDisplayer.displayConfirmationDialog();
        if(result.isPresent() && result.get().getText().equals("Yes")) {
            Platform.exit();
        }
    }
}