package bitsplease.mummymayhem.model.map;

import bitsplease.mummymayhem.model.buildings.Building;
import bitsplease.mummymayhem.model.buildings.Tower;
import bitsplease.mummymayhem.model.utils.Sprite;
import javafx.geometry.Point2D;

public class Field extends Sprite {
    FieldType type;
    Tower tower;
    Building building;

    public Field(Point2D position, FieldType type) {
        super(position, type.getImagePath());
        this.type = type;
        this.tower = null;
        this.building = null;
    }

    public FieldType getType() {
        return type;
    }

    public void setType(FieldType type) {
        this.type = type;
        this.imagePath = type.getImagePath();
    }

    public Tower getTower() { return tower; }

    public Building getBuilding() {
        return building;
    }

    /**
     * Adds a tower to the field.
     *
     * @param tower The tower to be added
     */
    public void addTower(Tower tower) {
        this.tower = tower;
    }

    /**
     * Removes the tower from the field.
     */
    public void removeTower() { this.tower = null; }

    /**
     * Adds a building to the field.
     *
     * @param building The building to be added
     */
    public void addBuilding(Building building) {
        this.building = building;
    }

    /**
     * Check if the field is occupied,
     * returns false if it's an open field, or it contains a tower or a building,
     * otherwise it returns true.
     *
     * @return If the field is occupied or not
     */
    public boolean isOccupied() {
        return (type != FieldType.OPEN || this.tower != null || this.building != null);
    }

    /**
     * Checks if the building contains a tower or not.
     *
     * @return If the field contains a tower or not
     */
    public boolean hasTower() {
        return tower != null;
    }

    public boolean hasBuilding() { return building != null; }
}
