package bitsplease.mummymayhem.model.persistence;

import bitsplease.mummymayhem.model.map.Field;
import bitsplease.mummymayhem.model.map.FieldType;
import bitsplease.mummymayhem.model.utils.DialogDisplayer;
import javafx.geometry.Point2D;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.*;

public class CustomMapPersistence implements GamePersistence {

    /**
     * Creates a file to the given path,
     * and saves the custom map's information in it.
     *
     * @param path The file path
     * @param save The CustomMapSave instance
     */
    @Override
    public void save(String path, CustomMapSave save) {
        FileWriter writer;
        try {
            File file = new File(path);
            writer = new FileWriter(file, false);
            writer.write(save.getWidth() + " ");
            writer.write(save.getHeight() + " ");
            writer.write("\n");
            for (int i = 0; i < save.getWidth(); i++) {
                StringBuilder line = new StringBuilder();
                for (int j = 0; j < save.getHeight(); j++) {
                    line.append(save.getFields().get(i).get(j).getType());
                    line.append(" ");
                }
                writer.write(line + "\n");
            }
            writer.close();
        } catch (IOException e) {
            DialogDisplayer.displayErrorDialog("An I/O error occurs during saving.");
        }
    }

    /**
     * Loads a file from the given path,
     * and returns a new CustomMapSave instance
     * with the appropriate fields.
     *
     * @param path The file path
     * @return The CustomMapSave instance
     */
    @Override
    public CustomMapSave load(String path) {
        CustomMapSave save = new CustomMapSave();
        Scanner sc;
        try {
            File file = new File(path);
            sc = new Scanner(file);

            List<List<Field>> fields = new ArrayList<>();
            int width = sc.nextInt();
            int height = sc.nextInt();
            for (int i = 0; i < width; i++) {
                List<Field> line = new ArrayList<>();
                for (int j = 0; j < height; j++) {
                    FieldType type = FieldType.valueOf(sc.next());
                    Field field = new Field(new Point2D(i, j), type);
                    line.add(field);
                }
                fields.add(line);
            }
            save = new CustomMapSave(width, height, fields);
        } catch (FileNotFoundException e) {
            DialogDisplayer.displayErrorDialog("File not found!");
            save.setFieldsEmpty();
        } catch (NoSuchElementException e) {
            DialogDisplayer.displayErrorDialog("Bad file format!");
            save.setFieldsEmpty();
        }
        return save;
    }
}
