package bitsplease.mummymayhem.model.units;

import bitsplease.mummymayhem.model.map.Field;
import bitsplease.mummymayhem.model.map.Map;
import bitsplease.mummymayhem.model.utils.ClimberPathFinder;
import bitsplease.mummymayhem.model.utils.PathFinder;
import javafx.geometry.Point2D;

import java.util.ArrayList;
import java.util.List;

public class DuneClimber extends Unit{
    private static final int BASE_HP=50;
    private static final int ATTACK_VALUE=7;
    private static final int SPEED_VALUE=3;
    private static final int PRICE=100;

    public DuneClimber(Point2D position, String imagePath) {
        super(position, imagePath, BASE_HP, SPEED_VALUE, ATTACK_VALUE, PRICE);
    }

    public DuneClimber(Unit unit) {
        super(unit);
    }

    public String getName() {
        return "Climber";
    }

    /**
     * Calculates and sets the default path without checkpoints,
     * only start position and destination are given.
     *
     * @param map The map object
     * @param destination The destination's position
     */
    @Override
    public void calculateDefaultPath(Map map, Point2D destination) {

        PathFinder pathFinder = new ClimberPathFinder(map.getWidth(), map.getHeight(),
                position, destination, map.getFields());
        List<Field> newPath = new ArrayList<>();

        for (Point2D p : pathFinder.getPath()) {
            newPath.add(map.getFields().get((int)p.getX()).get((int)p.getY()));
        }

        setPath(newPath);
    }

    /**
     * Can be called if the unit already has a default path.
     * Generate a new path using start position, destination
     * and checkpoints, then returns it in a list of positions.
     *
     * @param map The map object
     * @return The new path represented by a list of points
     */
    @Override
    public List<Point2D> getUpdatedPath(Map map) {

        List<Point2D> newPath = new ArrayList<>();
        Point2D from = this.position;
        Point2D destination = this.path.get(path.size()-1).getPosition();
        PathFinder pathFinder;

        for (Point2D checkPoint : this.checkPoints) {
            pathFinder = new ClimberPathFinder(map.getWidth(), map.getHeight(), from, checkPoint, map.getFields());
            newPath.addAll(pathFinder.getPath());
            newPath.remove(newPath.size()-1);
            from = checkPoint;
        }

        pathFinder = new ClimberPathFinder(map.getWidth(), map.getHeight(), from, destination, map.getFields());
        newPath.addAll(pathFinder.getPath());

        return newPath;

    }

    /**
     * Deletes checkpoints which became unreachable.
     *
     * @param map The map object
     */
    public void updateCheckpoints(Map map) {

        PathFinder pathFinder = new ClimberPathFinder(map.getWidth(), map.getHeight(),
                this.position, path.get(path.size() - 1).getPosition(),
                map.getFields());

        checkPoints.removeIf(checkpoint -> !pathFinder.isTraversable(checkpoint));
    }
}
