package bitsplease.mummymayhem.model.utils;

import javafx.scene.control.Alert;
import javafx.scene.control.ButtonBar;
import javafx.scene.control.ButtonType;
import javafx.stage.FileChooser;
import javafx.stage.Stage;

import java.io.File;
import java.util.Optional;

public class DialogDisplayer {

    /**
     * Creates a FileChooser for saving and shows it in the current stage.
     *
     * @param stage The current stage
     * @return The FileChooser instance
     */
    public static File displaySavingWindow(Stage stage) {
        FileChooser fileChooser = new FileChooser();
        fileChooser.setTitle("Save file");
        fileChooser.getExtensionFilters().add(new FileChooser.ExtensionFilter("Save files", "*.sav"));
        return fileChooser.showSaveDialog(stage);
    }

    /**
     * Creates a FileChooser for loading and shows it in the current stage.
     *
     * @param stage The current stage
     * @return The FileChooser instance
     */
    public static File displayLoadingWindow(Stage stage) {
        FileChooser fileChooser = new FileChooser();
        fileChooser.setTitle("Load file");
        fileChooser.getExtensionFilters().add(new FileChooser.ExtensionFilter("Save files", "*.sav"));
        return fileChooser.showOpenDialog(stage);
    }

    /**
     * Creates a new confirmation dialog and shows it.
     * @return An Optional that contains the alert result
     */
    public static Optional<ButtonType> displayConfirmationDialog() {
        Alert alert = new Alert(Alert.AlertType.CONFIRMATION);
        alert.setTitle("Exit Game");
        alert.setHeaderText(null);
        alert.setContentText("Are you sure?");

        ButtonType buttonTypeYes = new ButtonType("Yes", ButtonBar.ButtonData.YES);
        ButtonType buttonTypeCancel = new ButtonType("Cancel", ButtonBar.ButtonData.CANCEL_CLOSE);
        alert.getButtonTypes().setAll(buttonTypeYes, buttonTypeCancel);

        return alert.showAndWait();
    }

    /**
     * Creates a new error dialog with the given message and shows it.
     * @param message The error message
     */
    public static void displayErrorDialog(String message) {
        Alert alert = new Alert(Alert.AlertType.ERROR);
        alert.setTitle("Error");
        alert.setHeaderText("An error has appeared!");
        alert.setContentText(message);
        alert.show();
    }
}
