package bitsplease.mummymayhem.model.buildings;

import bitsplease.mummymayhem.model.map.Field;
import bitsplease.mummymayhem.model.map.FieldType;
import bitsplease.mummymayhem.model.units.Fighter;
import bitsplease.mummymayhem.model.units.Unit;
import javafx.geometry.Point2D;
import org.junit.jupiter.api.*;

import java.util.ArrayList;
import java.util.List;

public class TowerTest{
    Tower tower;

    @BeforeEach
    void setUp() {
        tower = new Tower(TowerType.BLOCKADE);
        tower.setPosition(new Point2D(0,0));
    }

    @Test
    void takeDamageTest(){
        tower.setHp(100);
        tower.takeDamage(10);
        Assertions.assertEquals(90, tower.getHp());
    }

    @Test
    void takeDamageTest2(){
        tower.setHp(5);
        tower.takeDamage(10);
        Assertions.assertEquals(0, tower.getHp());
        Assertions.assertTrue(tower.isDemolished());
    }

    @Test
    void attackTest(){
        Unit unit = new Fighter(new Point2D(0,0), "");

        unit.setHp(100);
        tower.setAttack(10);
        tower.attack(unit);

        Assertions.assertEquals(90, unit.getHp());
        Assertions.assertEquals(Tower.ATTACK_COOLDOWN_TIME, tower.getAttackCooldown());
        Assertions.assertEquals(tower.getProjectilePosition(), tower.getPosition());
    }

    @Test
    void attackTest2(){
        Unit unit = new Fighter(new Point2D(0,0), "");

        unit.setHp(5);
        tower.setAttack(10);
        tower.attack(unit);

        Assertions.assertEquals(0, unit.getHp());
        Assertions.assertEquals(Tower.ATTACK_COOLDOWN_TIME, tower.getAttackCooldown());
        Assertions.assertEquals(tower.getProjectilePosition(), tower.getPosition());
    }

    @Test
    void upgradeTest() {
        tower.setHp(100);
        tower.setAttack(5);
        tower.setRange(1);
        tower.upgrade();

        Tower tower2 = new Tower(TowerType.DESTROYER);
        tower2.setHp(100);
        tower2.setAttack(5);
        tower2.setRange(1);
        tower2.upgrade();

        Tower tower3 = new Tower(TowerType.GUARDIAN);
        tower3.setHp(100);
        tower3.setAttack(5);
        tower3.setRange(1);
        tower3.upgrade();

        Assertions.assertEquals(2, tower.getLevel());
        Assertions.assertEquals(100+Tower.HP_INCREASE_PER_UPGRADE, tower.getHp());
        Assertions.assertEquals(5, tower.getAttack());
        Assertions.assertEquals(1, tower.getRange());

        Assertions.assertEquals(2, tower2.getLevel());
        Assertions.assertEquals(100, tower2.getHp());
        Assertions.assertEquals(5 + Tower.ATTACK_INCREASE_PER_UPGRADE, tower2.getAttack());
        Assertions.assertEquals(1, tower2.getRange());

        Assertions.assertEquals(2, tower3.getLevel());
        Assertions.assertEquals(100, tower3.getHp());
        Assertions.assertEquals(5, tower3.getAttack());
        Assertions.assertEquals(1 + Tower.RANGE_INCREASE_PER_UPGRADE, tower3.getRange());

        tower.upgrade();
        tower2.upgrade();
        tower3.upgrade();

        Assertions.assertEquals(3, tower.getLevel());
        Assertions.assertEquals(100+Tower.HP_INCREASE_PER_UPGRADE*2, tower.getHp());
        Assertions.assertEquals(5, tower.getAttack());
        Assertions.assertEquals(1, tower.getRange());

        Assertions.assertEquals(3, tower2.getLevel());
        Assertions.assertEquals(100, tower2.getHp());
        Assertions.assertEquals(5 + Tower.ATTACK_INCREASE_PER_UPGRADE*2, tower2.getAttack());
        Assertions.assertEquals(1, tower2.getRange());

        Assertions.assertEquals(3, tower3.getLevel());
        Assertions.assertEquals(100, tower3.getHp());
        Assertions.assertEquals(5, tower3.getAttack());
        Assertions.assertEquals(1 + Tower.RANGE_INCREASE_PER_UPGRADE*2, tower3.getRange());

        tower.upgrade();
        tower2.upgrade();
        tower3.upgrade();

        Assertions.assertEquals(3, tower.getLevel());
        Assertions.assertEquals(100+Tower.HP_INCREASE_PER_UPGRADE*2, tower.getHp());
        Assertions.assertEquals(5, tower.getAttack());
        Assertions.assertEquals(1, tower.getRange());

        Assertions.assertEquals(3, tower2.getLevel());
        Assertions.assertEquals(100, tower2.getHp());
        Assertions.assertEquals(5 + Tower.ATTACK_INCREASE_PER_UPGRADE*2, tower2.getAttack());
        Assertions.assertEquals(1, tower2.getRange());

        Assertions.assertEquals(3, tower3.getLevel());
        Assertions.assertEquals(100, tower3.getHp());
        Assertions.assertEquals(5, tower3.getAttack());
        Assertions.assertEquals(1 + Tower.RANGE_INCREASE_PER_UPGRADE*2, tower3.getRange());

        Tower demolishedTower = new Tower(TowerType.BLOCKADE);
        demolishedTower.setHp(100);
        demolishedTower.setAttack(5);
        demolishedTower.setRange(1);
        demolishedTower.setDemolished(true);
        demolishedTower.upgrade();

        Assertions.assertEquals(1, demolishedTower.getLevel());
        Assertions.assertEquals(100, demolishedTower.getHp());
        Assertions.assertEquals(5, demolishedTower.getAttack());
        Assertions.assertEquals(1, demolishedTower.getRange());
    }

    @Test
    void targetUnitTest() {
        List<Unit> units = new ArrayList<>();
        List<Field> path1 = new ArrayList<>();
        List<Field> path2 = new ArrayList<>();
        List<Field> path3 = new ArrayList<>();

        path1.add(new Field(new Point2D(0,0) ,FieldType.OPEN));
        path1.add(new Field(new Point2D(0,0) ,FieldType.OPEN));
        path2.add(new Field(new Point2D(0,0) ,FieldType.OPEN));
        path3.add(new Field(new Point2D(0,0) ,FieldType.OPEN));

        Unit unit1 = new Fighter(new Point2D(0,0), "");
        Unit unit2 = new Fighter(new Point2D(0,0), "");
        Unit unit3 = new Fighter(new Point2D(3,7), "");

        unit1.setPath(path1);
        unit2.setPath(path2);
        unit3.setPath(path3);

        units.add(unit1);
        units.add(unit2);
        units.add(unit3);

        tower.targetUnit(units);
        Assertions.assertEquals(unit2, tower.getTarget());
    }
}