package bitsplease.mummymayhem.model.units;

import bitsplease.mummymayhem.model.map.Field;
import bitsplease.mummymayhem.model.map.FieldType;
import bitsplease.mummymayhem.model.map.Map;
import javafx.geometry.Point2D;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.List;
import java.util.Random;

import static org.junit.jupiter.api.Assertions.*;

class DuneClimberTest {
    Unit duneClimber;

    @BeforeEach
    void setUp() {
        duneClimber = new DuneClimber(new Point2D(0,0), "src/main/resources/assets/units/duneClimber/");
    }

    @Test
    void calculateDefaultPathTest() {
        Map map = new Map(10,10);
        duneClimber.setPosition(map.getPlayerACastlePosition());

        duneClimber.calculateDefaultPath(map, map.getPlayerBCastlePosition());

        assertTrue(duneClimber.getPath().size() > 0);

        for(Field f: duneClimber.getPath()) {
            assertTrue(f.getType().equals(FieldType.OPEN) || f.getType().equals(FieldType.DUNE));
        }
    }

    @Test
    void getUpdatedPathTest() {
        Map map = new Map(10,10);
        duneClimber.setPosition(map.getPlayerACastlePosition());

        int c = 0;
        for (List<Field> fields : map.getFields()) {
            for (Field field : fields) {
                if(c == 0) {
                    field.setType(FieldType.OPEN);
                    c++;
                }
                else {
                    field.setType(FieldType.DUNE);
                    c--;
                }
            }
        }

        duneClimber.calculateDefaultPath(map, map.getPlayerBCastlePosition());
        Random rand = new Random();

        // Generates 3 random checkpoints
        for(int i = 0; i < 3; i++) {
            int randX;
            int randY;
            do {
                randX = rand.nextInt(10);
                randY = rand.nextInt(10);

            } while((map.getField(randY, randX).getType()!= FieldType.OPEN
                    && map.getField(randY, randX).getType()!= FieldType.DUNE)
                    || map.isCastlePosition(randX,randY)
                    || duneClimber.getCheckPoints().contains(new Point2D(randX,randY)));

            duneClimber.addCheckPoint(new Point2D(randX, randY));
        }


        assertTrue(duneClimber.getPath().size() > 0);

        int cpInUpdatedPath = 0;

        for(Point2D p: duneClimber.getUpdatedPath(map)) {
            assertTrue(map.getField((int)p.getY(), (int)p.getX()).getType().equals(FieldType.OPEN)
                    || map.getField((int)p.getY(), (int)p.getX()).getType().equals(FieldType.DUNE));
            assertFalse(map.getField((int)p.getY(), (int)p.getX()).hasTower());

            for (Point2D cp: duneClimber.getCheckPoints()) {
                if(p.equals(cp)) {
                    cpInUpdatedPath++;
                }
            }
        }

        assertTrue(cpInUpdatedPath >= 3);
    }
}