package bitsplease.mummymayhem.model.utils;

import bitsplease.mummymayhem.model.buildings.Barrack;
import bitsplease.mummymayhem.model.map.Field;
import bitsplease.mummymayhem.model.map.FieldType;
import bitsplease.mummymayhem.model.map.Map;
import javafx.geometry.Point2D;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.Collection;
import java.util.List;

public class PathFinderTest {

    Map map;
    PathFinder pathFinder;

    @BeforeEach
    void setUp() {
        map = new Map(10,10);
        pathFinder = new PathFinder(map.getWidth(), map.getHeight(),
                                    map.getPlayerACastlePosition(), map.getPlayerBCastlePosition(),
                                    map.getFields());

        for (int i = 0; i < map.getWidth(); i++) {
            for (int j = 0; j < map.getHeight(); j++) {
                if (map.getPlayerACastlePosition() != map.getFields().get(i).get(j).getPosition()
                        && map.getPlayerBCastlePosition() != map.getFields().get(i).get(j).getPosition()
                        &&  j != 4 && j != 3 && j != 2) {
                    map.setField(i, j, FieldType.TREE);
                }
                else {
                    map.setField(i, j, FieldType.OPEN);
                }
            }
        }
        pathFinder.refresh(map.getFields());
    }

    @Test
    void isTraversableTest() {
        Assertions.assertTrue(pathFinder.isTraversable());
    }

    @Test
    void isTraversableTest2() {
        Field openField = map.getFields().stream().flatMap(Collection::stream)
                .filter(field -> !field.isOccupied())
                .findAny().get();

        Assertions.assertTrue(pathFinder.isTraversable(openField.getPosition()));
    }

    @Test
    void isTraversableTest3() {

        Field occupiedField = map.getFields().stream().flatMap(Collection::stream)
                .filter(field -> field.isOccupied() && field.getPosition() != map.getPlayerBCastlePosition())
                .findAny().get();

        Assertions.assertFalse(pathFinder.isTraversable(occupiedField.getPosition()));
    }

    @Test
    void blockPathTest() {
        for (int i = 0; i < map.getWidth(); i++) {
            for (int j = 0; j < map.getHeight(); j++) {
                if (map.getPlayerACastlePosition() != map.getFields().get(i).get(j).getPosition()
                    && map.getPlayerBCastlePosition() != map.getFields().get(i).get(j).getPosition()
                    &&  j != 4) {
                    map.setField(i, j, FieldType.TREE);
                }
                else {
                    map.setField(i, j, FieldType.OPEN);
                }
            }
        }
        pathFinder.refresh(map.getFields());
        Assertions.assertTrue(pathFinder.blockPath(new Point2D(4, 4)));
    }

    @Test
    void refreshTest() {
        Field buildableField = map.getFields().stream().flatMap(Collection::stream)
                .filter(field -> !field.isOccupied() && !pathFinder.blockPath(field.getPosition()))
                .findAny().get();

        Assertions.assertTrue(pathFinder.isTraversable(buildableField.getPosition()));
        buildableField.addBuilding(new Barrack(buildableField.getPosition()));
        pathFinder.refresh(map.getFields());
        Assertions.assertFalse(pathFinder.isTraversable(buildableField.getPosition()));
    }

    @Test
    void removeFromNodesTest() {
        Field openField = map.getFields().stream().flatMap(Collection::stream)
                .filter(field -> !field.isOccupied() && !pathFinder.blockPath(field.getPosition()))
                .findAny().get();

        pathFinder.removeFromNodes(openField.getPosition());

        Assertions.assertAll(
                () -> Assertions.assertFalse(pathFinder.isTraversable(openField.getPosition())),
                () -> Assertions.assertTrue(pathFinder.isTraversable())
        );
    }

    @Test
    void getPathTest() {
        Assertions.assertAll(
                () -> Assertions.assertTrue(pathFinder.getPath().contains(new Point2D(4, 4))),
                () -> Assertions.assertTrue(pathFinder.isTraversable(new Point2D(2, 2))),
                () -> Assertions.assertFalse(pathFinder.getPath().contains(new Point2D(2, 2))),
                () -> Assertions.assertEquals(11, pathFinder.getPath().size())
        );

        List<Point2D> shortestPath = List.of(
                new Point2D(0,4),
                new Point2D(1,4),
                new Point2D(2,4),
                new Point2D(3,4),
                new Point2D(4,4),
                new Point2D(5,4),
                new Point2D(6,4),
                new Point2D(7,4),
                new Point2D(8,4),
                new Point2D(9,4),
                new Point2D(9,5));

        List<Point2D> pathFinderPath = pathFinder.getPath();
        for (int i = 0; i<shortestPath.size(); i++) {
            Assertions.assertEquals(shortestPath.get(i), pathFinderPath.get(i));
        }
    }
}
